;; -*- Lisp -*-
;; $Id: $
;; absolute-darkness/theme.jl
;; by damian christey

;; based on Absolute S : Copyright (C) 1999 Mike Bond <mbond@erols.com>
;; now based more on AquaX:  Copyright (C) 2000 Kevin D. Knerr, Sr. <ld_barthel@yahoo.com>
;; with some pieces of "Operational" by Juliano Soares dos Santos (juliano@eps.ufsc.br)

;; Inspired by Hallvar Helleseth's Absolute E theme for Enlightenment. 
;; hallvar@ii.uib.no (www.ii.uib.no/~hallvar)

;; Images stolen from DarkAlloy theme for Enlightenment. :)
;; scarab@egyptian.net (www.egyptian.net/~scarab)

;;    Date: Nov. 16, 2000.
;; All that we see or seem is but a dream within a dream.

(define theme-name 'absolute-darkness)

;; preferences

(defgroup absolute-darkness "Absolute Darkness Theme"
  :group appearance)

(defcustom absolute-darkness:foreground "white"
  "Title foreground color"
  :group (appearance absolute-darkness)
  :type color)

(defcustom absolute-darkness:full-title nil
  "Use full width of window for title and buttons"
  :group (appearance absolute-darkness)
  :type boolean)

(defcustom absolute-darkness:shrink-shaped-title nil
  "Shrink title to width of title text when shaded"
  :group (appearance absolute-darkness)
  :type boolean)

(defcustom absolute-darkness:title-align 'Left
  "Title bar alignment"
  :group (appearance absolute-darkness)
  :type symbol
  :options (Left Right))

(defcustom absolute-darkness:border-dimension 4
  "Width of window border"
  :group (appearance absolute-darkness)
  :type number
  :range (1 . 16))

(defcustom absolute-darkness:use-icons nil
  "Use application icon for titlebar menu button."
  :group (appearance absolute-darkness)
  :type boolean)

;; new frame part classes

(def-frame-class do-nothing '((cursor . left_ptr)))

;; functions

(define title-width
  (lambda (w)
    (let
	((w-width (car (window-dimensions w))))
      (cond ((eq absolute-darkness:full-title nil)
	     (max 0 
		  (min (- w-width 64) 
		       (max (- (/ w-width 2) 
			       64) 
			    (+ 10 
			       (text-width 
				(window-name w)))))))
	    ((eq absolute-darkness:full-title t)
	     (max 0 (- w-width 64)))))))

(define title-width-shaped 
  (lambda (w)
    (let
	((w-width (car (window-dimensions w))))
      (cond ((eq absolute-darkness:shrink-shaped-title nil)
	     (title-width w))
	    ((eq absolute-darkness:shrink-shaped-title t)
	     (max 0 (min (- w-width 64) (+ 10 (text-width (window-name w))))))))))

(define border-dimension (lambda () absolute-darkness:border-dimension))
(define -border-dimension (lambda () (- absolute-darkness:border-dimension)))
(define foreground (lambda () absolute-darkness:foreground))
    
;; Make images

(define bar-images (mapcar (lambda (i)
			     (set-image-border i 4 4 4 4))
			   (list (make-image "bar_normal.png")
				 (make-image "bar_normal_active.png")
				 (make-image "bar_hilited_active.png")
				 (make-image "bar_clicked_active.png"))))

(define close-images (mapcar (lambda (i)
			       (set-image-border i 4 4 4 4))
			     (list (make-image "close_normal.png")
				   (make-image "close_normal_active.png")
				   (make-image "close_hilited_active.png")
				   (make-image "close_clicked_active.png"))))

(define iconify-images (mapcar (lambda (i)
				 (set-image-border i 4 4 4 4))
			       (list (make-image "iconify_normal.png")
				     (make-image "iconify_normal_active.png")
				     (make-image "iconify_hilited_active.png")
				     (make-image "iconify_clicked_active.png"))))

(define maximize-images (mapcar (lambda (i)
				  (set-image-border i 4 4 4 4))
				(list (make-image "maximize_normal.png")
				      (make-image "maximize_normal_active.png")
				      (make-image "maximize_hilited_active.png")
				      (make-image "maximize_clicked_active.png"))))

(define menu-images (mapcar (lambda (i)
			      (set-image-border i 4 4 4 4))
			    (list (make-image "menu_normal.png")
				  (make-image "menu_normal_active.png")
				  (make-image "menu_hilited_active.png")
				  (make-image "menu_clicked_active.png"))))

(define bg-images (mapcar (lambda (i)
			      (set-image-border i 4 4 4 4))
			    (list (make-image "bg_normal.png")
				  (make-image "bg_normal_active.png")
				  (make-image "bg_hilited_active.png")
				  (make-image "bg_clicked_active.png"))))

(define default-icon (make-image "default_icon.png"))

(define bg-btn-icon (lambda (w)
		      (let ((icon (window-icon-image w)))
			(if (eq icon nil) default-icon icon))))

;;;;
;;;; We can define frame component groups that will not change here
;;;;

;; border groups
(define normal-border-group 
  `(;; right frame
    ((background . ,bar-images)
     (right-edge . ,-border-dimension)
     (width . ,border-dimension)
     (top-edge . 0)
     (bottom-edge . 0)
     (class . right-border))
    ;; left frame
    ((background . ,bar-images)
     (left-edge . ,-border-dimension)
     (width . ,border-dimension)
     (top-edge . 0)
     (bottom-edge . 0)
     (class . left-border))
    ;; bottom frame
    ((background . ,bar-images)
     (left-edge . 0)
     (right-edge . 0)
     (bottom-edge . ,-border-dimension)
     (height . ,border-dimension)
     (class . bottom-border))
    ;; top-left corner
    ((background . ,bar-images)
     (left-edge . ,-border-dimension)
     (width . ,border-dimension)
     (top-edge . ,-border-dimension)
     (height . ,border-dimension)
     (class . top-left-corner))
    ;; top-right corner
    ((background . ,bar-images)
     (right-edge . ,-border-dimension)
     (width . ,border-dimension)
     (top-edge . ,-border-dimension)
     (height . ,border-dimension)
     (class . top-right-corner))
    ;; bottom-left corner
    ((background . ,bar-images)
     (left-edge . ,-border-dimension)
     (width . ,border-dimension)
     (bottom-edge . ,-border-dimension)
     (height . ,border-dimension)
     (class . bottom-left-corner))
    ;; bottom-right corner
    ((background . ,bar-images)
     (right-edge . ,-border-dimension)
     (width . ,border-dimension)
     (bottom-edge . ,-border-dimension)
     (height . ,border-dimension)
     (class . bottom-right-corner))
    ))

(define transient-top-group 
  `(;; top frame
    ((background . ,bar-images)
     (left-edge . 0)
     (right-edge . 0)
     (top-edge . ,-border-dimension)
     (height . ,border-dimension)
     (class . top-border))
    ))


;;;;
;;;; Functions to alter button placement and titlebar dimensions
;;;;

;; For simplicity, we only setq the buttons needed for each style.
;;   Therefore, to avoid void-value errors, we need to define them
;;   all in advance (although I can think of no circumstance in
;;   which we would *not* define the titlebar positions)

(define close-button-position 0)
(define iconify-button-position 0)
(define maximize-button-position 0)
(define menu-button-position 0)
(define shaped-menu-button-position 0)
(define top-border-right 0)
(define top-border-left 0)
(define title-position 0)
(define menu-button-background 0)

;; First a function to place our buttons
(define set-button-placement
  (lambda ()
    (case absolute-darkness:title-align
      ((Left)
       (setq iconify-button-position `(left-edge . 0))
       (setq maximize-button-position `(left-edge . 16))
       (setq menu-button-position `(left-edge . 32))
       (setq title-position `(left-edge . 48))
       (setq close-button-position `(left-edge .  ,(lambda (w) (+ (title-width w) 48))))
       (setq shaped-menu-button-position `(left-edge . 32))
       (setq top-border-left `(left-edge .  ,(lambda (w) (+ (title-width w) 64))))
       (setq top-border-right `(right-edge . 0)) 
       )
      ((Right)
       (setq close-button-position `(right-edge . 0))
       (setq title-position `(right-edge . 16))
       (setq menu-button-position `(right-edge .  ,(lambda (w) (+ (title-width w) 16))))
       (setq maximize-button-position `(right-edge .  ,(lambda (w) (+ (title-width w) 32))))
       (setq iconify-button-position `(right-edge .  ,(lambda (w) (+ (title-width w) 48))))
       (setq shaped-menu-button-position `(right-edge . ,(lambda (w) (+ (title-width-shaped w) 16))))
       (setq top-border-right `(right-edge .  ,(lambda (w) (+ (title-width w) 64))))
       (setq top-border-left `(left-edge . 0))
       )      
      )
    )
  )
;; use icons or not
(define set-icons
  (lambda ()
    (cond ((eq absolute-darkness:use-icons nil)
	   (setq menu-button-background `(background . ,menu-images))
	   )
	  ((eq absolute-darkness:use-icons t)
	   (setq menu-button-background `(background . ,bg-btn-icon))
	   )
	  )
    )
  )
   
;;;;
;;;; Combine the components into groups
;;;;

(define normal-top-group nil)
(define shaped-top-group nil)

(define set-frame-groups
  (lambda ()
    (setq normal-top-group
	  `(;; close button
	    ((background . ,close-images)
	     ,close-button-position
	     (width . 16)
	     (top-edge . -16)
	     (height . 16)
	     (class . close-button))
	    ;; title bar
	    ((background . ,bar-images)
	     (foreground . ,foreground)
	     (text . ,window-name)
	     (x-justify . center)
	     (y-justify . center)
	     ,title-position
	     (width . ,title-width)
	     (top-edge . -16)
	     (class . title))
	    ;; iconify button
	    ((background . ,iconify-images)
	     ,iconify-button-position
	     (width . 16)
	     (top-edge . -16)
	     (height . 16)
	     (class . iconify-button))
	    ;; maximize button
	    ((background . ,maximize-images)
	     ,maximize-button-position
	     (width . 16)
	     (top-edge . -16)
	     (height . 16)
	     (class . maximize-button))
	    ;; behind menu button
	    ((background . ,bg-images)
	     ,menu-button-position
	     (width . 16)
	     (height . 16)
	     (top-edge . -16)
	     (class . do-nothing))
	    ;; menu button
	    (,menu-button-background
	     ,menu-button-position
	     (width . 16)
	     (height . 16)
	     (top-edge . -16)
	     (class . menu-button))
	    ;; rhs bit of title
	    ((background . ,bar-images)
	     ,top-border-right
	     ,top-border-left
	     (top-edge . ,-border-dimension)
	     (height . ,border-dimension)
	     (class . top-border))
	    ))

    (setq shaped-top-group
	  `(;; close button
	    ((background . ,close-images)
	     ,close-button-position
	     (width . 16)
	     (top-edge . -16)
	     (height . 16)
	     (class . close-button))
	    ;; title bar
	    ((background . ,bar-images)
	     (foreground . ,foreground)
	     (text . ,window-name)
	     (x-justify . center)
	     (y-justify . center)
	     ,title-position
	     (width . ,title-width-shaped)
	     (top-edge . -16)
	     (cursor . left_ptr)
	     (class . title))
	    ;; behind menu button
	    ((background . ,bg-images)
	     ,shaped-menu-button-position
	     (width . 16)
	     (height . 16)
	     (top-edge . -16)
	     (class . do-nothing))
	    ;; menu button
	    ((background . ,menu-button-background)
	     ,shaped-menu-button-position
	     (width . 16)
	     (height . 16)
	     (top-edge . -16)
	     (class . menu-button))
	    ))
    ))

;;;;
;;;; Finally, we create our full frame definitions
;;;;

(define normal-frame nil)
(define shaped-frame nil)
(define transient-frame nil)
(define shaped-transient-frame nil)

(define create-frames
  (lambda ()
    ;; First, call any functions that set values we need to create our frames
    (set-button-placement)
    (set-icons)
    (set-frame-groups)
  
    ;; Now we create the various frames required for our theme
    (setq normal-frame (append
      normal-top-group
      normal-border-group
    ))
    
    (setq shaped-frame (append
      shaped-top-group
    ))
    
    (setq transient-frame (append
      transient-top-group
      normal-border-group
    ))
    
    (setq shaped-transient-frame (append
      transient-top-group
    ))
  )
)  

(create-frames)

;; At last! We create the actual theme
(add-frame-style theme-name
  (lambda (w type)
    (case type
      ((default)              normal-frame)
      ((shaped)               shaped-frame)
      ((transient)            transient-frame)
      ((shaped-transient)     shaped-transient-frame)
      ((unframed)             nil-frame)
    )
  )
)

;;;;
;;;; Define some functions to handle frame updates correctly
;;;;

(define (reframe-one w)
  (if (eq (window-get w 'current-frame-style) theme-name)
    (rebuild-frame w)
  )
)

(define reframe-all
  (lambda ()
    (create-frames)
    (reframe-windows-with-style 'absolute-darkness)
  )
) 

;;;;
;;;; Set conditions for calling the frame update functions
;;;;

(call-after-property-changed 'WM_NAME reframe-one)
(call-after-property-changed 'WM_HINTS	reframe-one)
(call-after-state-changed '(sticky) reframe-one)
;; (call-after-state-changed '(shaded) reframe-one)

;;;;
;;;; Define the after-set properties of our custom variables
;;;;

(mapc (lambda (sym)
	(custom-set-property sym ':after-set reframe-all))
      '(absolute-darkness:foreground
	absolute-darkness:full-title
	absolute-darkness:shrink-shaped-title
	absolute-darkness:title-align
	absolute-darkness:border-dimension
	absolute-darkness:use-icons))

